;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Loading this file installs CG/JS.
;; Please see readme.txt for more information
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(in-package :user)

(eval-when (compile eval load)
  (require :shell)
  (use-package :excl.shell))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; validate files

(let ((platform-file
       (directory
	(merge-pathnames
	 "PLATFORM-*"
	 (path-pathname *load-pathname*)))))
  (or platform-file
      (error "Could not find the PLATFORM file."))
  
  (when (> (length platform-file) 1)
    (error "There is more than one PLATFORM file: ~a." platform-file))
  (setq platform-file (car platform-file))
  
  (or (=~ "^PLATFORM-(.*)" (file-namestring platform-file))
      (error "Cannot parse the PLATFORM file."))
  
  (let ((expected-platform $1)
	(current-platform
	 (format nil "~a~a" (symbol-name sys::.os.) #+smp "smp" #-smp "")))
    (or (string-equal expected-platform current-platform)
	(error
	 "The current platform (~a) does not match the downloaded version (~a)."
	 current-platform expected-platform)))
  ;; We are on the correct platform, fall through...
  )

;; HTML files needed for the operation of CG/JS
;; (this variable's value is generated by build_cgjs.sh)
(defvar *html-files* '("cgjs.html" "ssl-not-found.html" "browser-not-supported.html" ))

(defvar *extra-files*)
(setq *extra-files* nil)

(dolist (file *html-files*)
  (setq file (merge-pathnames file *load-pathname*))
  (or (probe-file file)
      (error "~a does not exist." file))
  (push file *extra-files*))
;; All *html-files* exist...

;; These two are directories but the pathname is specified as
;; a file, so merging works below.
(defvar *cg2js-directory* (merge-pathnames "code.cg2js" *load-pathname*))
(defvar *idejs-directory* (merge-pathnames "code.idejs" *load-pathname*))

(dolist (d (list *cg2js-directory* *idejs-directory*))
  (or (probe-file d)
      (error "~a does not exist." d)))

;; MD5 checksums of the incoming files
;; (this variable's value is generated by build_cgjs.sh)
(defvar *checksums* 
'(
(#x86b37bd8a2bda053a7330ab371ea4a9f . "./code.cg2js/cg-status-bar.fasl")
(#x5eb5b38310de0638d8ab040c94a5ea5e . "./code.cg2js/cg-hotspot.fasl")
(#x317dfb5c7090ddd539762b44abea1a03 . "./code.cg2js/cg-toolbar.fasl")
(#xa1be46d904df1041787e7f544bf42730 . "./code.cg2js/cg-clipboard-stack.fasl")
(#x87c23f8599fd16b227647b3158e6c02f . "./code.cg2js/cg-tooltip.fasl")
(#x146cf2f0904496ae339417526f7c8b46 . "./code.cg2js/cg-calendar.fasl")
(#xec8af8350b3733bff8b64c2aba315b48 . "./code.cg2js/cg-item-list.fasl")
(#xb79427f2db876924b92942edfbbf1a27 . "./code.cg2js/cg-bitmap-stream.fasl")
(#xcff607dde480c51af74d4a78fbff0d30 . "./code.cg2js/cg-trackbar.fasl")
(#x9b260985a71da840c5859a9f7f0fc934 . "./code.cg2js/cg-class-grid.fasl")
(#xa5ed5b69e5dcf0c92b1d5880e66d9b74 . "./code.cg2js/cg-dialog-item.fasl")
(#xa52cc7af9cba42f5ba078fcd02de2fa2 . "./code.cg2js/cg-comtab.fasl")
(#x9456181ed9e73e4392b5741d7ea92cc3 . "./code.cg2js/cg-lisp-text.fasl")
(#xd07593e9f5ab519ede7214d1765a56f1 . "./code.cg2js/cg-rich-edit-pane.fasl")
(#x25f4120496e1a40a81a341747e836087 . "./code.cg2js/cg-button.fasl")
(#x2e4c14b434c22bbc3758a6b70613f40b . "./code.cg2js/cg-printing.fasl")
(#x5914fe2185cb65270729e43289b5df8c . "./code.cg2js/cg-pixmap-printing.fasl")
(#x21d5cfd70999411324f356d34d428159 . "./code.cg2js/cg-multi-picture-button-tooltip.fasl")
(#xe6ae8e58c3e7a8789f4bd38457446e31 . "./code.cg2js/cg-pixmap-widget.fasl")
(#xbf70c6a8918267215091eb1dfbe51e89 . "./code.cg2js/cg-selected-object.fasl")
(#x65b790e5e0a1935c992810cc9fca80dd . "./code.cg2js/cg-text-or-combo.fasl")
(#x041192bbd7312938da88c4be6dcbcce1 . "./code.cg2js/cg-graphics-context.fasl")
(#xc90af79a4482049d4e21b303948a1aa5 . "./code.cg2js/cg-edit-in-place.fasl")
(#x011e879c271a18a28cfbe2c8c1042ee0 . "./code.cg2js/cg-string-dialog.fasl")
(#x4111d405bb2f7fbaca834e751e2f3ed7 . "./code.cg2js/cg-list-view.fasl")
(#x979cb7978967aa22269cadae3deada9c . "./code.cg2js/cg-intersect.fasl")
(#x9cf6c715109062c0cdc493c7806a17c4 . "./code.cg2js/cg-text-widget.fasl")
(#x8d0a786dbfeef703404d93431cf060a9 . "./code.cg2js/cg-directory-dialog.fasl")
(#x9d4aab8f9c7d0b544563e4f076686674 . "./code.cg2js/cg-file-dialog.fasl")
(#xf7cfbe8faf88803de2edb90a0788d3bf . "./code.cg2js/cg-dde-utils.fasl")
(#xb47544530141911f92129df208107960 . "./code.cg2js/cg-nodes.fasl")
(#x712873c606c36ed43949ed0d1305b4be . "./code.cg2js/cg-text-edit-pane-mark.fasl")
(#xa60d9acbe2a0a2dd72fcbd02ea32d31c . "./code.cg2js/cg-up-down-control.fasl")
(#x1fdc34eed34cd3a39f27f98655068b0d . "./code.cg2js/cg-get-position.fasl")
(#xdf28008b48ddfeb99868a045d5ef23e8 . "./code.cg2js/cg-toggling-widget.fasl")
(#x652c61912372a05fdf701b06265dd670 . "./code.cg2js/cg-acache.fasl")
(#x259175d30316767d159f8ae3ccdbc7e8 . "./code.cg2js/cg-outline-drag-and-drop.fasl")
(#x85ae60db8eb5fba1855974a33e58d1ee . "./code.cg2js/cg-base.fasl")
(#x3abe5a725cc4a1d33f447e7baf223eb5 . "./code.cg2js/cg-drag-and-drop.fasl")
(#x7c6094f35339648569e7762b92761b87 . "./code.cg2js/cg-find-string-dialog.fasl")
(#x7505ea284ccb326bc10bdddce39081d7 . "./code.cg2js/cg-property.fasl")
(#x268c90ae48b70b6fa75b4f4230130741 . "./code.cg2js/cg-editable-text.fasl")
(#xc1a7d1a33189121de9bfe19e2c8e7b52 . "./code.cg2js/cg-lamp.fasl")
(#x54cbf868482824fc52683a1e1e288dc7 . "./code.cg2js/cg-progress-indicator.fasl")
(#xb45059ebd76ac133ad69aea8a6950dec . "./code.cg2js/cg-chart-widget.fasl")
(#xdc869d79b743124daf4030024f6ff396 . "./code.cg2js/cg-common-control.fasl")
(#xe6dd5539cee05ee442d80f88cb21060c . "./code.cg2js/cg-font-dialog.fasl")
(#x2afcd26f2e690ed424a0d05efe2a1b15 . "./code.cg2js/cg-gesture-emulation.fasl")
(#x9814c9f0ad6bd635fd177cd59e9ec44e . "./code.cg2js/cg-pixmap-file-io.fasl")
(#x42f16c0829aaeb96f1e714193d583a71 . "./code.cg2js/cg-os-window.fasl")
(#xd3de36857a1e2031b6c282fab7b27785 . "./code.cg2js/cg-multi-line-lisp-text.fasl")
(#x28db871a93fa679a8be47dda25507791 . "./code.cg2js/cg-pixmap.fasl")
(#x929e0aca6d45cf8b91593f6f8a01ff17 . "./code.cg2js/cg-chart-or-plot.fasl")
(#x06ece967ac83f9e8f5169372ae0261f9 . "./code.cg2js/cg-scroll-bar-mixin.fasl")
(#x26bb1a3bcdbce063982cf72a7e857dcd . "./code.cg2js/cg-scaling-stream.fasl")
(#x6b0c892077d8a2559c8536799bf9281b . "./code.cg2js/cg-grid-widget-drag-and-drop.fasl")
(#xba2627906a932d5e558827ef9aa006cc . "./code.cg2js/cg-picture-widget-palette.fasl")
(#x2884137af256e544548782e1b7b15828 . "./code.cg2js/cg-timer.fasl")
(#x9f3a040995472a1592d0aac871c37959 . "./code.cg2js/cg-scroll-bar.fasl")
(#xf275c1ec88e733645268abf66177c8a1 . "./code.cg2js/cg-caret.fasl")
(#xadba6ea26d0f28620b2ea2dbf411ad7d . "./code.cg2js/cg-template-string.fasl")
(#xfbb44fee1808433238da454d5cd4ce0d . "./code.cg2js/cg-multi-picture-button-drag-and-drop.fasl")
(#x5edbde34302d14cd8a7838890a815f06 . "./code.cg2js/cg-directory-dialog-os.fasl")
(#xb3708d1268bb890a070d04aba1f7205a . "./code.cg2js/cg-rich-edit.fasl")
(#x2ac62f2929cfeab81b828aaf018a90fe . "./code.cg2js/cg-combo-box.fasl")
(#x2c158fc3435c6869483eaace4e45016e . "./code.cg2js/cg-text-edit-pane.fasl")
(#x7e4964c6fd24194c899fc02b28eac137 . "./code.cg2js/cg-multi-picture-button.fasl")
(#x1ae77bf62428f9a6f6e620c9593c3c95 . "./code.cg2js/cg-menu-tooltip.fasl")
(#x524541d1250c1f874712dfebac4139d1 . "./code.cg2js/cg-scrolling-static-text.fasl")
(#x47bdf7c05636a64c20d9fc0c80120975 . "./code.cg2js/cg-outline.fasl")
(#xd085ee6859889d97960c6461b444999d . "./code.cg2js/cg-class-support.fasl")
(#x94a676a67ee9807d817ea5bd88f65305 . "./code.cg2js/cg-lettered-menu.fasl")
(#x7f8b1975e34ad4799a2989ee88eecb20 . "./code.cg2js/cg-cursor-pixmap.fasl")
(#x90f13ce7f1a86e519010ef12c8f5b1e6 . "./code.cg2js/cg-fill-texture.fasl")
(#xc05dec8832367086fa888446d59e8218 . "./code.cg2js/cg-lisp-widget.fasl")
(#xdbfa57b90f3a84c1908eab59c4bb6c46 . "./code.cg2js/cg-keyboard-shortcuts.fasl")
(#x4015f2863bbbf634f5d356a126819670 . "./code.cg2js/cg-clipboard.fasl")
(#x39701aad094d24e0a64266d9531a2dbd . "./code.cg2js/cg-wrap-string.fasl")
(#xdad9a905c05265dafd1d2b32847768e7 . "./code.cg2js/cg-os-widget.fasl")
(#x318f60195c76afa478fc77a0c9c1b2da . "./code.cg2js/cg-check-box.fasl")
(#x5fa7fc3a12c01d754a7a7f82c44ebed3 . "./code.cg2js/cg-choice-list.fasl")
(#xa0fd73bc69bdef6f32b1c451b0cc1589 . "./code.cg2js/cg-layout.fasl")
(#x22556424253a1f9d3d5ccaad880c227a . "./code.cg2js/cg-group-box.fasl")
(#xc3980d11991b3e4667304049bc02a663 . "./code.cg2js/cg-bitmap-pane-clipboard.fasl")
(#x53e50241126801952b37d9ee851f9f5d . "./code.cg2js/cg-radio-button.fasl")
(#x4addf7e8470bd7442510cf8665d99480 . "./code.cg2js/cg-tab-control.fasl")
(#x0675634475acdf0f3059fafec57cb927 . "./code.cg2js/cg.fasl")
(#x043192f1dd280342e4f223524b194299 . "./code.cg2js/cg-drawable-clipboard.fasl")
(#xcf32db04cfd4d1d83192a7252c53c682 . "./code.cg2js/cg-dropping-outline.fasl")
(#xa2ce82b40cdf0154635e072cee363f39 . "./code.cg2js/cg-get-pixmap.fasl")
(#x953d9a8cc3b19e0931b9bf39ffb696ec . "./code.cg2js/cg-grid-widget.fasl")
(#x221e0132ffcf38762b31a23eb0ddca4a . "./code.cg2js/cg-utility-dialog.fasl")
(#x71c50ae657b50bc51aae52558a991ca9 . "./code.cg2js/cg-static-text.fasl")
(#xb79f2df9c7d34fcc1aca3707dae7c326 . "./code.cg2js/cg-class-slot-grid.fasl")
(#x4e41efd941b0d0af50da2a00d4416078 . "./code.cg2js/cg-object-editor-layout.fasl")
(#xcab0581279f9dbe9fb072ad1b8b6a62f . "./code.cg2js/cg-drag-and-drop-image.fasl")
(#x0694816e92a19f699561aa2280f4f377 . "./code.cg2js/cg-bitmap-pane.fasl")
(#x7f4a6e070e9018f9cd00111d774fd737 . "./code.cg2js/cg-plot-widget.fasl")
(#x1b6a09c31a00e1c702228dee3ba919d9 . "./code.cg2js/cg-color-dialog.fasl")
(#xa6533f333c976661063f83fdbb2a3e63 . "./code.cg2js/cg-icon-pixmap.fasl")
(#x603b195c6ac63dd96194b5d6fd0ff7bc . "./code.cg2js/cg-icon.fasl")
(#x5cf23b9e9b3410389445c01c245e870c . "./code.cg2js/cg-dev.fasl")
(#x23f9b92e01392b1f9d4c6b9dccbec5d9 . "./code.cg2js/cg-choose-printer.fasl")
(#x2ae981c088a474d6029fd7f6a59301cb . "./code.cg2js/cg-intersect-posbox.fasl")
(#x765a9e2cabe751170dc956e33b249751 . "./code.cg2js/cg-menu.fasl")
(#x42f9ce842afc8b64854a887e3a955145 . "./code.cg2js/cg-yes-no-list.fasl")
(#xd5c7e0f8b8e78cc6cf5bf9746160c062 . "./code.cg2js/cg-palette.fasl")
(#xddcc2de4566cba7320ec50b01ef9f571 . "./code.cg2js/cg-outline-edit-in-place.fasl")
(#xc4c55eedbb363245e9f90d2047dc1e82 . "./code.cg2js/cg-message-dialog.fasl")
(#x0ca137b7138101ca1313009ed90bd2b5 . "./code.cg2js/cg-shortcut-menu.fasl")
(#xe14c66bd9b5a579a21b9c4cbc5b89d64 . "./code.cg2js/cg-rich-edit-pane-clipboard.fasl")
(#x902bc6d6d37ecc95fc6a0fafa3b287ce . "./code.cg2js/cg-split-bar.fasl")
(#x8fe9a317f49b5d171830c618e77204c8 . "./code.cg2js/cg-multi-line-editable-text.fasl")
(#xd57c29ae0b32b8737788640b0f4c68e1 . "./code.cg2js/cg-web-browser.fasl")
(#x4e5ca3812466a18854a43c8379b396e3 . "./code.cg2js/cg-project-window.fasl")
(#x8ed103293ac8c1ecac3b714a46417a7d . "./code.cg2js/cg-picture-widget.fasl")
(#x192b2d9b7da2bd68f2a444380faaa2ef . "./code.cg2js/cg-pixmap-rotate.fasl")
(#xe21c0b8857a93f5217eec80181cf35b2 . "./code.cg2js/cg-object-editor.fasl")
(#xdfc939a33d642e668e9d038487896322 . "./code.cg2js/cg-rich-edit-pane-printing.fasl")
(#x5e3926aad4d1832c72597f0c7963aa26 . "./code.cg2js/cg-paren-matching.fasl")
(#x080c20dcd9a9df38f4bb3d21c345eeb2 . "./code.cg2js/cg-sample-file-menu.fasl")
(#x865fe440d183524e90b2134da7655711 . "./code.cg2js/cg-drawable.fasl")
(#x73fb93b440ae867598582eec42f11d90 . "./code.cg2js/cg-text-edit-pane-file-io.fasl")
(#xd8f77b748991e3b74548d2c0559b00b8 . "./code.cg2js/cg-html-widget.fasl")
(#x5f3a762219d9d106d3721389d67ce0aa . "./code.cg2js/cg-curve.fasl")
(#x37706cb21599f5e2caa497be0507bfc5 . "./code.cg2js/cg-yes-no-string.fasl")
(#x4ad08cd6128acd572341018207fa2327 . "./code.cg2js/cg-lisp-edit-pane.fasl")
(#x9797db2aa248bca24b64cbf96bc328b8 . "./code.cg2js/cg-html-dialog.fasl")
(#x5de0461e052ad7c260dfcf4ef982e82d . "./code.cg2js/cg-clipboard-pixmap.fasl")
(#x92f0c42dcea777a0851119950333bd12 . "./code.cg2js/cg-tray.fasl")
(#x40d10e1c5f9bed2af559069644c4f0c8 . "./code.idejs/ide-apropos.fasl")
(#x3bc648395a298ee69c426c61e173f00f . "./code.idejs/ide-window-list.fasl")
(#x50b776e3ac9a60804077098a1d479e4d . "./code.idejs/ide-class-browser.fasl")
(#x0917d00d1190c1af4ad979c62fb5edff . "./code.idejs/ide-base.fasl")
(#xc0a36b7ada3722568df24d9602e2b105 . "./code.idejs/ide-process-browser.fasl")
(#x81a9c0114029c17e83978bf8f2cddc3a . "./code.idejs/ide-acache.fasl")
(#x6f68932420b9a59f17e8375384005a81 . "./code.idejs/ide-navigator.fasl")
(#xb18d005a864140c81c09e56eeeb6d8e9 . "./code.idejs/ide-trace.fasl")
(#x28c171a14889549fb4e9c47594665739 . "./code.idejs/ide-project.fasl")
(#x7be958d19d835af3f2907b4bc26b95bb . "./code.idejs/ide-patch-dialog.fasl")
(#xc05d16235af534b62a1dada33e9bf09e . "./code.idejs/ide-grapher.fasl")
(#xc44a2a0995af9c1baa8c7634ab4667ea . "./code.idejs/ide-dev.fasl")
(#x013a78f2e8e813967fc152f309a9e597 . "./code.idejs/ide-keyshort.fasl")
(#x842832e6467085850a3ca089612cf340 . "./code.idejs/ide-editor.fasl")
(#xc2733f674d64eba202894ede21304a51 . "./code.idejs/ide-inspector.fasl")
(#xe6afde42a92c13af57ea4882888f2cfe . "./code.idejs/ide.fasl")
(#xb94b68836b77c1d8fad8e1a0a1a4d94f . "./code.idejs/ide-clipboard.fasl")
(#x70188c0d529651d1c7bf3ebacb74c9fc . "./code.idejs/ide-find-definitions.fasl")
(#x258bd44fd3c228f592586965d1d53ba0 . "./code.idejs/ide-form.fasl")
(#x98a02199b1714a3c9136fa3a31110aa7 . "./code.idejs/ide-intro.fasl")
(#x9f5ab33e02c2afd9a27d7b1464cfd328 . "./code.idejs/ide-find-in-files.fasl")
(#x98ce7a48eed842f70cdccfa5e8a440af . "./code.idejs/ide-package-list.fasl")
(#xd5a4ac77258e4f16000f1233d7e94a3c . "./code.idejs/ide-class-grapher.fasl")
(#x696b64509ebe0538a3b58ac1081d4fa0 . "./cgjs.html")
(#x34779e5d4bbb71d45bf63f88fac54100 . "./ssl-not-found.html")
(#x90c6d2a65504d45266ad9fa3a49be2f9 . "./browser-not-supported.html"))
 )

(let ((files-ok
       (dolist (checksum *checksums* t)
	 (when (/= (car checksum)
		   (md5-file
		    (merge-pathnames (cdr checksum) *load-pathname*)))
	   (return nil)))))
  (cond
   ((not files-ok)
    (error "The extracted files failed the checksum test.  ~
Please download and/or extract again."))
   (t
    (format t "Checksums on files are OK...~%"))))

;; We have validated all the files are there...

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; check for needed patches

(eval-when (compile eval load) (require :build))

(defvar *required-patches* '(#+macosx ("build" . 3)))

(dolist (required-patch *required-patches*)
  (let ((existing-patch (assoc (car required-patch)
			    (cdr (assoc :system sys::*patches*))
			    :test #'string=)))
    (when (null existing-patch)
      (error "Could not find patch info for the ~s module."
	     (car required-patch)))
    (when (not (>= (second existing-patch) (cdr required-patch)))
      ;; The build patch number is NOT >= to the one we require
      (error "~
Expected version ~d of ~s but have version ~d.  Please update ~
your lisp with update.~a and then do the CG/JS installation again."
	     (cdr required-patch) (car required-patch)
	     (second existing-patch)
	     #+mswindows "exe"
	     #-mswindows "sh"))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; copy files

(defun copy (source dest)
  (cond
   ((file-directory-p source)
    (format t "Copy directory ~a~%" (enough-namestring source))
    (copy-directory source dest :quiet t))
   (t
    (format t "Copy file ~a~%" (enough-namestring source))
    (sys:copy-file source dest :verbose nil))))

(defvar *sys* (translate-logical-pathname (pathname "sys:")))

(cond
 ((excl::pathname-equalp (truename *sys*)
			 (path-pathname *load-truename*))
  (format t "Files are already in place, no copying requiring.~%"))
 (t ;; copy files, replacing them with new ones if they already exist
  (let (dest)
    (dolist (from *extra-files*)
      (setq dest (merge-pathnames (file-namestring from) *sys*))
      (when (probe-file dest) (delete-file dest))
      (copy from dest))
    
    (dolist (from (list *cg2js-directory* *idejs-directory*))
      (setq dest (merge-pathnames (file-namestring from) *sys*))
      ;; now make it a directory pathname
      (setq dest (pathname-as-directory dest))

      (when (probe-file dest) (delete-directory-and-files dest))
      (copy (pathname-as-directory from) dest)))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; update develenv.cl to load CG/JS before the original version of CG

(defvar *setup-cgjs-loading* "sys:setup-cgjs-loading.cl")

;; Because users might use an ANSI lisp to install, have to make sure
;; the resulting file can be loaded into an ANSI and Modern lisp.
(with-open-file (s *setup-cgjs-loading* :direction :output
		 :if-exists :supersede)
  (format s "~
;; This file was created by install-cgjs.cl.
;; Please do not edit it.

")
  (dolist (dir (list "code.cg2js/" "code.idejs/"))
    (format s "
(let ((path (make-pathname
	     :type \"fasl\"
	     :directory (pathname-directory
			 (merge-pathnames
			  ~s
			  (translate-logical-pathname \"sys:\"))))))
  (format t \";; *require-search-list*: adding: ~~s~%\" path)
  (push path sys:*require-search-list*))

"
	    dir)))

(defvar *develenv.cl* (pathname "sys:develenv.cl"))

(when (not (probe-file *develenv.cl*))
  (setf (file-contents *develenv.cl*)
    ";; This file is loaded before images are rebuilt with update.sh/update.exe

"))

(let ((contents (file-contents *develenv.cl*)))
  (cond
   ((match-re
     (format nil "^#\\|\\|# \\(load \"sys:~a"
	     (file-namestring *setup-cgjs-loading*))
     contents
     :multiple-lines t)
    (format t "develenv.cl already correctly loads CG/JS.~%"))
   ((match-re
     (format nil "^#\\+ignore \\(load \"sys:~a"
	     (file-namestring *setup-cgjs-loading*))
     contents
     :multiple-lines t)
    (format t "develenv.cl: CG/JS disabled; will enable.~%")
    (setf (file-contents *develenv.cl*)
      (replace-re  contents
		  (format nil "^#\\+ignore *\\(load \"sys:~a"
			  (file-namestring *setup-cgjs-loading*))
		  (format nil "#||# (load \"sys:~a"
			  (file-namestring *setup-cgjs-loading*))
		  :multiple-lines t
		  :count 1)))
   (t
    (format t "develenv.cl: adding CG/JS initialization.~%")
    (setf (file-contents *develenv.cl*)
      (concatenate 'simple-string
	contents
	(format nil "~

;; To disable the use of CG/JS and go back to the originally released
;; (and possibly patched) version of CG/IDE, change the #||# to
;; #+ignore on the following line and the rebuild images by running
;; update.sh or update.exe, depending on your platform:
#||# (load \"sys:~a\")~%~%"
		(file-namestring *setup-cgjs-loading*)))))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; success!

(let ((msg (format nil "~
CG/JS installation successful!

NOTE: to use CG/JS you must rebuild images by running 'update.~a'
      in the Allegro CL directory (~a)."
		   #+mswindows "exe"
		   #-mswindows "sh"
		   *sys*)))
  #-mswindows (format t "~&~%~a~%" msg)
  #+mswindows (excl::internal-message-box msg "CG/JS Installation" :type :ok))
